@@ -1,10 +1,10 @@
 /*
  * scrm is an implementation of the Sequential-Coalescent-with-Recombination Model.
- * 
+ *
  * Copyright (C) 2013, 2014 Paul R. Staab, Sha (Joe) Zhu and Gerton Lunter
- * 
+ *
  * This file is part of scrm.
- * 
+ *
  * scrm is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -30,21 +30,21 @@ using namespace Rcpp;
 
 class RRandomGenerator : public RandomGenerator {
  public:
-  RRandomGenerator() {
+  RRandomGenerator(FastFunc* ff):RandomGenerator(ff) {
     this->initializeUnitExponential();
   };
   virtual ~RRandomGenerator() {};
 
   void initialize() {};
-  
-  double sample() { 
+
+  double sample() {
     RNGScope scope;
     return R::runif(0,1);
   }
-  
+
   double sampleUnitExponential() {
     RNGScope scope;
-    return R::rexp(1); 
+    return R::rexp(1);
   }
 };
 
