@@ -40,7 +40,11 @@ void Forest::initialize(Model* model,
 
   this->set_model(model);
   this->set_random_generator(rg);
-  this->set_current_base(1);
+
+  current_rec_ = 0;
+  rec_bases_ = std::vector<double>(1, -1);
+  rec_bases_.reserve(1000);
+
   this->set_sample_size(0);
 
   this->coalescence_finished_ = true;
@@ -75,9 +79,8 @@ Forest::Forest(const Forest &current_forest) {
 
   // Copy state information
   this->set_sample_size(current_forest.sample_size());
-  this->set_current_base(current_forest.current_base());
-  this->set_next_base(current_forest.next_base());
-  this->segment_count_ = current_forest.segment_count_;
+  this->rec_bases_ = current_forest.rec_bases_;
+  this->current_rec_ = current_forest.current_rec_;
 
   // Copy the nodes
   this->nodes_ = NodeContainer(*current_forest.getNodes());
@@ -126,7 +129,7 @@ Node* Forest::cut(const TreePoint &cut_point) {
   if ( !cut_point.base_node()->local() )
     new_leaf->make_nonlocal(cut_point.base_node()->last_update());
   else
-    new_leaf->make_nonlocal(current_base());
+    new_leaf->make_nonlocal(current_rec());
   assert( !new_leaf->local() );
 
   new_leaf->set_population(cut_point.base_node()->population());
@@ -197,7 +200,7 @@ void Forest::updateAbove(Node* node, bool above_local_root,
   // Fast forward above local root because this part is fairly straight forward
   if (above_local_root) {
     // Assure that everything is non-local
-    if (node->local()) node->make_nonlocal(current_base());
+    if (node->local()) node->make_nonlocal(current_rec());
 
     // Update the primary root if needed
     if ( node->is_root() ) {
@@ -210,7 +213,7 @@ void Forest::updateAbove(Node* node, bool above_local_root,
     return;
   }
 
-  node->set_last_change(current_base());
+  node->set_last_change(current_rec());
 
   // Calculate new values for samples_below and length_below for the current
   // node
@@ -237,10 +240,10 @@ void Forest::updateAbove(Node* node, bool above_local_root,
   // Update whether the node is local or not 
   if (!invariants_only) {
     if (samples_below == 0) {
-      if ( node->local() ) node->make_nonlocal(current_base());
+      if ( node->local() ) node->make_nonlocal(current_rec());
     }
     else if ( samples_below == sample_size() ) {
-      if ( node->local() ) node->make_nonlocal(current_base());
+      if ( node->local() ) node->make_nonlocal(current_rec());
 
       // Are we the local root?
       if (node->countChildren() == 2 && 
@@ -278,9 +281,12 @@ void Forest::updateAbove(Node* node, bool above_local_root,
  */
 void Forest::buildInitialTree() {
   dout << "===== BUILDING INITIAL TREE =====" << std::endl;
-  assert( this->nodes()->size() == 0 );
-  this->set_current_base(0.0);
-  this->segment_count_ = 1;
+  assert(this->nodes()->size() == 0);
+  assert(this->segment_count() == 0);
+  assert(this->rec_bases_.size() == 1);
+  assert(this->model().getCurrentSequencePosition() == 0.0);
+  this->set_next_base(0.0);
+  ++current_rec_;
 
   dout << "* Adding first node... ";
   Node* first_node = nodes()->createNode(model().sample_time(0), 1);
@@ -314,6 +320,7 @@ void Forest::buildInitialTree() {
     assert(this->printNodes());
   }
   this->sampleNextBase();
+  dout << "Next Sequence position: " << this->next_base() << std::endl;
   this->calcSegmentSumStats();
 }
 
@@ -404,24 +411,26 @@ TreePoint Forest::samplePoint(Node* node, double length_left) {
  * @ingroup group_pf_update
  */
 void Forest::sampleNextGenealogy() {
-  this->set_current_base(next_base_);
+  ++current_rec_; // Move to next recombination;
 
-  if (current_base_ == model().getCurrentSequencePosition()) {
+  if (current_base() == model().getCurrentSequencePosition()) {
     // Don't implement a recombination if we are just here because rates changed
     dout << std::endl << "Position: " << this->current_base() << ": Changing rates." << std::endl;
     this->sampleNextBase();
     this->calcSegmentSumStats();
+    dout << "Next Position: " << this->next_base() << std::endl;
     return;
   }
 
-  dout << tmp_event_time_ << std::endl;
+  assert( current_base() > model().getCurrentSequencePosition() );
+  assert( current_base() < model().getNextSequencePosition() );
+
   assert( tmp_event_time_ >= 0 );
   this->contemporaries_.buffer(tmp_event_time_);
 
-  ++segment_count_;
   dout << std::endl << "===== BUILDING NEXT GENEALOGY =====" << std::endl;
-  dout << "Segment Nr: " << segment_count_ << std::endl;
-  dout << "Sequence position: " << this->current_base() << std::endl;
+  dout << "Segment Nr: " << current_rec() << " | "
+       << "Sequence position: " << this->current_base() << std::endl;
   assert( this->current_base() <= this->model().loci_length() );
 
   // Sample the recombination point
@@ -444,10 +453,8 @@ void Forest::sampleNextGenealogy() {
   assert( this->printTree() );
   assert( this->printNodes() );
 
-  // Rarely prune all possible nodes 
-  if (segment_count_ % 100000 == 0 && model().exact_window_length() != -1) this->doCompletePruning();
-
   this->sampleNextBase();
+  dout << "Next Sequence position: " << this->next_base() << std::endl;
   this->calcSegmentSumStats();
 }
 
@@ -470,7 +477,7 @@ void Forest::sampleCoalescences(Node *start_node) {
   coalescence_finished_ = false;
 
   // This assertion needs an exception for building the initial tree
-  assert ( segment_count_ == 1 || 
+  assert ( current_rec() == 1 || 
            active_node(1)->in_sample() || 
            start_node->height() <= active_node(1)->height() );
 
@@ -613,6 +620,10 @@ void Forest::calcRates(const TimeInterval &ti) {
     // recombining
     rates_[0] += calcRecombinationRate(active_node(1));
   }
+
+  assert(rates_[0] >= 0);
+  assert(rates_[1] >= 0);
+  assert(rates_[2] >= 0);
 }  
 
 
@@ -767,10 +778,13 @@ double Forest::calcPwCoalescenceRate(const size_t pop, const TimeInterval &ti) c
 
 
 double Forest::calcRecombinationRate(Node const* node) const {
-  assert( !node->local() );
-  if (node->last_update() >= model().getCurrentSequencePosition()) {
+  assert(!node->local());
+  assert(this->current_base() >= model().getCurrentSequencePosition());
+  double last_update_pos = get_rec_base(node->last_update());
+
+  if (last_update_pos >= model().getCurrentSequencePosition()) {
     // Rec rate is constant for the relevant sequence part
-    return ( model().recombination_rate() * (this->current_base() - node->last_update()) );
+    return (this->current_base() - last_update_pos) * model().recombination_rate();
   } else {
     // Rec rate may change. Accumulate the total rate.
 
@@ -778,13 +792,13 @@ double Forest::calcRecombinationRate(Node const* node) const {
         (this->current_base() - model().getCurrentSequencePosition()); 
     size_t idx = model().get_position_index() - 1;
 
-    while (model().change_position(idx) > node->last_update()) {
+    while (model().change_position(idx) > last_update_pos) {
       assert(idx > 0);
       rate += model().recombination_rate(idx) * (model().change_position(idx+1)-model().change_position(idx));
       --idx;
     }
 
-    rate += model().recombination_rate(idx) * (model().change_position(idx+1)-node->last_update());
+    rate += model().recombination_rate(idx) * (model().change_position(idx+1)-last_update_pos);
     return rate;
   }
 }
@@ -1132,7 +1146,7 @@ Node* Forest::possiblyMoveUpwards(Node* node, const TimeInterval &time_interval)
 
 bool Forest::pruneNodeIfNeeded(Node* node, const bool prune_orphans) {
   assert(node != NULL);
-  if (model().exact_window_length() == -1) return false;
+  if (!model().has_approximation()) return false;
   if (node->in_sample()) return false;
 
   if (node->is_root()) {
@@ -1206,14 +1220,6 @@ void Forest::printLocusSumStats(std::ostream &output) const {
   }
 }
 
-void Forest::doCompletePruning() {
-  dout << "Pruning Tree:" << std::endl;
-  for (NodeIterator ni = nodes()->iterator(nodes()->first()->next()); ni.good(); ++ni) {
-    pruneNodeIfNeeded((*ni)->previous());
-  }
-  pruneNodeIfNeeded(nodes()->last());
-}
-
 void Forest::clear() {
   // Clear roots tracking
   set_local_root(NULL);
@@ -1223,11 +1229,16 @@ void Forest::clear() {
   nodes()->clear();
 
   // Reset Position & Segment Counts
-  this->set_current_base(0.0);
-  this->segment_count_ = 0;
+  this->rec_bases_.clear();
+  this->set_next_base(-1.0);
+  this->current_rec_ = 0;
 
   // Clear Summary Statistics
   this->clearSumStats();
+  
+  // Reset Model
+  writable_model()->resetTime();
+  writable_model()->resetSequencePosition();
 }
 
 
@@ -1276,7 +1287,7 @@ Node* Forest::readNewickNode( std::string &in_str, std::string::iterator &it, si
     } else if ( (*(it)) == ';' ) {
       dout <<" Node " << node << " closed " << std::endl;
       this->nodes()->add( node );
-      node->make_nonlocal( this->current_base() );
+      node->make_nonlocal(current_rec());
       return node;
     } else {
       continue;
@@ -1288,8 +1299,7 @@ Node* Forest::readNewickNode( std::string &in_str, std::string::iterator &it, si
 
 
 void Forest::readNewick( std::string &in_str ){
-  this->set_current_base(0.0);
-  this->segment_count_ = 1;
+  this->current_rec_ = 1;
   std::string::iterator it = in_str.begin();
   (void)this->readNewickNode( in_str, it );
   this->set_local_root( this->nodes()->last() );
@@ -1301,8 +1311,24 @@ void Forest::readNewick( std::string &in_str ){
   }
   assert(this->printNodes());
   assert(this->printTree());
-dout << "contemporaries_.size()"<<contemporaries_.size(0) <<std::endl;
+  dout << "contemporaries_.size()"<<contemporaries_.size(0) <<std::endl;
   this->sampleNextBase();
   this->calcSegmentSumStats();
   this->tmp_event_time_ = this->local_root()->height();
 }
+
+
+// Must be called AFTER the tree was modified.
+void Forest::sampleNextBase() {
+  double length = random_generator()->sampleExpoLimit(getLocalTreeLength() * model().recombination_rate(),
+                                                      model().getNextSequencePosition() - current_base());
+  if (length == -1) {
+    // No recombination until the model changes
+    set_next_base(model().getNextSequencePosition());
+    if (next_base() < model().loci_length()) writable_model()->increaseSequencePosition();
+  } else {
+    // Recombination in the sequence segment
+    set_next_base(current_base() + length);
+  }
+} 
+
